/*+++*
 *  title:	BrowserHelper.h
 *  abstract:	Interface of BrowserHelper support class.
 *  author:	Tom Hageman <tom@basil.icce.rug.nl>
 *  created:	March 1999
 *  modified:	(see RCS Log at end)
 *  copyright:
 *
 *		Copyright (C) 1999  Tom R. Hageman.
 *
 *	This is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This software is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this software; see the file COPYING.  If not, write to
 *	the Free Software Foundation, 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 *
 *  description:
 *
 *      This module defines the following classes:
 *      - BrowserHelper
 *
 *	This class implements incremental search and name completion
 *	in the associated browser, improved arrow-key navigation,
 *	and support for adding/deleting entries.
 *
 *	Features:
 *
 *	- incremental search: typing into the associated text field scrolls
 *        to the next best matching cell in the browser.  Works best when
 *	  the contents of the browser is sorted case-insensitive alphabetically.
 *
 *	- completion: the Escape key completes whetever is in the text field
 *	  from the currently selected cell in the browser.
 *
 *	- augmented Up/Down Arrow key navigation: Up/Down select the
 *	  previous/next cell without wraparound; [Shift] Up/Down Arrow select
 *	  to the previous/next cell with ``meaningful'' contents (as defined
 *	  by the client code) or non-leaf cell; [Alternate] and [Control]
 *	  Up/Down select the first/last cell. (both are provided since on
 *	  Intel, with NeXTXyzzy keyboard mapping, the [Alt]-arrow keys
 *	  produced rubbish characters - probably because Alt was remapped to
 *	  the Control key, that produced the same rubbish in combination with
 *	  the arrow keys...)
 *---*/

#ifdef RCS_BrowserHelper_ID
static const char * const RCS_h_id = ((void)&RCS_h_id,
	"@(#)BrowserHelper.h,v 1.3 1999/04/18 08:23:05 tom Exp");
#endif

#import <appkit/appkit.h>

@interface BrowserHelper : Object
{
    id	addButton;
    id	browser;
    id	removeButton;
    id	textField;

    BOOL sendActionOnSelect;
    SEL cellHasContentsSelector;    // to find cell with shift-arrow-up/down.
}

- (NXBrowser *)browser;
- (TextField *)textField;
- (Button *)addButton;
- (Button *)removeButton;

- (BOOL)sendActionOnSelect;
- (void)setSendActionOnSelect:(BOOL)aValue;

- (SEL)cellHasContentsSelector;
- (void)setCellHasContentsSelector:(SEL)aCellHasContentsSelector;
    /* this is used to determine whether a cell has (meaningful) contents,
       so [Shift] Arrow up/down key handler can jump to/between them. */

- add:sender;
- remove:sender;

- reload:sender;
- select:sender;

@end // BrowserHelper


@interface Object (BrowserHelperDelegate)

/* Additional browser delegate methods (invoked by BrowserHelper). */

- (BOOL)browser:(NXBrowser *)aBrowser willAddCellNamed:(const char *)aName
    inMatrix:(Matrix *)aMatrix atRow:(int)aRow;
    /* invoked in -add:, just before the cell is added.  Delegate can add
       a new cell itself either at the suggested place in the matrix, or
       somewhere else.  If delegate does not implement this method, or
       this method returns YES, BrowserHelper will add a new cell itself.
       The suggested place tries hard to keep the list in case-insensitive
       alphabetical order. */

- (void)browser:(NXBrowser *)aBrowser didAddCell:(id)aCell
    inMatrix:(Matrix *)aMatrix atRow:(int)aRow;
    /* invoked in -add:, just after the cell is added. */

- (BOOL)browser:(NXBrowser *)aBrowser willRemoveCellInMatrix:(Matrix *)aMatrix
    atRow:(int)aRow;
    /* invoked in -remove:, just before the cell is removed.  Delegate
       can remove the cell itself, or just refuse to remove it.  If
       delegate does not implement this method, or this method returns
       YES, BrowserHelper will remove the cell itself. */

- (void)browser:(NXBrowser *)aBrowser didRemoveCellInMatrix:(Matrix *)aMatrix
    atRow:(int)aRow;
    /* invoked in -remove:, just after the cell has been removed. */

@end // Object (BrowserHelperDelegate)
