/*+++
 *  title:      BrowserHelper.m
 *  abstract:   Implementation of BrowserHelper support class.
 *  author:     Tom Hageman <tom@basil.icce.rug.nl>
 *  created:    March 1999
 *  modified:   (see RCS Log at end)
 *  copyleft:
 *
 *		Copyright (C) 1999  Tom R. Hageman.
 *
 *	This is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This software is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this software; see the file COPYING.  If not, write to
 *	the Free Software Foundation, 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 *
 *  description:
 *      (see corresponding *.h file)
 *---*/

#ifdef RCS_ID
static const char * const RCSid = ((void)&RCSid,
	"@(#)BrowserHelper.m,v 1.2 1999/04/17 21:22:23 tom Exp");
#   define RCS_BrowserHelper_ID
#endif

#import "BrowserHelper.h"


#define ESC	'\033'

/* The following is is nasty little hack to morph the class of an existing
   instance into something else entirely.  Don't try this at home, kids. */

static void changeClassOfInstanceTo(id instance, Class class)
{
   if (instance) instance->isa = class;
}

/* Yet another nasty little hack, to notify a TextField's delegate of some
   additional special characters. */

#ifdef BrowserHelper // has been aliased -- derive support class name from alias
# undef CAT
# undef _CAT
# define CAT(x,y)	_CAT(x,y)
# define _CAT(x,y)	x ## y
# define BrowserHelperTextFieldCell  CAT(BrowserHelper,TextFieldCell)
#endif

@interface BrowserHelperTextFieldCell : TextFieldCell
@end

@implementation BrowserHelperTextFieldCell

static id editor;

static unsigned short
fieldFilter(unsigned short theChar, int flags, unsigned short charSet)
{
    unsigned short result = NXFieldFilter(theChar, flags, charSet);
    id delegate;

#if DEBUG
    fprintf(stderr, "fieldFilter(%#x, %#x, %u) = %#x\n", theChar, flags, charSet, result);
#endif
    switch (result)
    {
      case 0:
	if (theChar != ESC) break;
	result = theChar;
      case NX_UP:
      case NX_DOWN:
	// Abuse -textDidEnd:endChar: to signal delegate.
	delegate = [[editor delegate] textDelegate];
	if ([delegate respondsTo:@selector(textDidEnd:endChar:)])
	{
	    [delegate textDidEnd:editor endChar:result];
	}
	if (result == ESC) result = NX_RIGHT;  // Duh!  Just to avoid bleep.
    }
    return result;
}

static void installFieldFilter(id textObj)
{
    editor = textObj;
    [textObj setCharFilter:fieldFilter];
}

static void removeFieldFilter(id textObj)
{
    editor = nil;
    [textObj setCharFilter:NXFieldFilter];
}

- select:(const NXRect *)aRect inView:controlView editor:textObj delegate:anObject start:(int)selStart length:(int)selLength
{
    id result = [super select:aRect inView:controlView editor:textObj delegate:anObject start:selStart length:selLength];
#if DEBUG
    fprintf(stderr, "-[%s %s]\n", [[self class] name], sel_getName(_cmd));
#endif
    installFieldFilter(textObj);
    return result;
}

- edit:(const NXRect *)aRect inView:controlView editor:textObj delegate:anObject event:(NXEvent *)theEvent
{
    id result = [super edit:aRect inView:controlView editor:textObj delegate:anObject event:theEvent];
#if DEBUG
    fprintf(stderr, "-[%s %s]\n", [[self class] name], sel_getName(_cmd));
#endif
    installFieldFilter(textObj);
    return result;
}

- endEditing:textObj
{
#if DEBUG
    fprintf(stderr, "-[%s %s]\n", [[self class] name], sel_getName(_cmd));
#endif
    removeFieldFilter(textObj);
    return [super endEditing:textObj];
}

@end // BrowserHelperTextFieldCell


@interface BrowserHelper (Private)
- (void)addCellNamed:(const char *)aName inMatrix:(Matrix *)aMatrix atRow:(int)aRow;
- (void)selectItemInBrowser:(const char *)anItem;
- (void)handleArrowKey:(unsigned short)key;
- (void)complete:sender;
@end


@implementation BrowserHelper (Private)

- init
{
    id instance = [super init];

    if (instance == self)
    {
	sendActionOnSelect = YES;   // for backward compatibility.
    }
    return instance;
}

- awakeFromNib
{
    /* Setup browser to accept arrow keys, and setup associated textField
       to forward arrow key events to the browser. */
    [browser acceptArrowKeys:YES andSendActionMessages:[self sendActionOnSelect]];
    changeClassOfInstanceTo([textField cell], [BrowserHelperTextFieldCell class]);
    return self;
}

- (void)addCellNamed:(const char *)aName inMatrix:(Matrix *)aMatrix atRow:(int)aRow
{
    id cell;
    
    [aMatrix insertRowAt:aRow];
    cell = [aMatrix cellAt:aRow :0];	
    [cell setStringValue:aName];
    [cell setLoaded:YES];
    [cell setLeaf:YES];
}

- (void)selectItemInBrowser:(const char *)anItem
{
    const char *bestMatch = "";
    BOOL exactMatch = NO;

    if (anItem[0])
    {
	Matrix *matrix = [browser matrixInColumn:[browser lastColumn]];
	int i, cellCount = [matrix cellCount];
	int bestMatchLen = -1;

	// Search best match -- OPTIM: use binary search.
	for (i = cellCount;  --i >= 0; )
	{
	    const char *cellTitle = [[matrix cellAt:i:0] stringValue];
	    int j = 0;

	    while (NXToLower(cellTitle[j]) == NXToLower(anItem[j]) && anItem[j]) ++j;

	    if (j < bestMatchLen) continue;

	    if (j == bestMatchLen &&
		NXToLower(cellTitle[j]) < NXToLower(anItem[j])) continue;

	    bestMatchLen = j;
	    bestMatch = cellTitle;

	    if (anItem[j] == '\0' && cellTitle[j] == '\0')
	    {
		// Check again, case-sensitively, to see if exact match.
		exactMatch = (strcmp(cellTitle, anItem) == 0);
		if (exactMatch) break;
	    }
	}
#if DEBUG
	fprintf(stderr, "-selectItemInBrowser:\"%s\" - bestMatch = \"%s\" (exact = %s, i = %d, count = %d)\n", anItem, bestMatch, exactMatch ? "YES" : "NO", i, cellCount);
#endif
    }
    // Select item in browser, and send action to update detail.
    [browser setPath:bestMatch];
    if ([self sendActionOnSelect]) [browser sendAction];

    // Enable Add button depending on current value.
    [addButton setEnabled:!(anItem[0] == '\0' || exactMatch)];
    [removeButton setEnabled:exactMatch];
}

- (void)handleArrowKey:(unsigned short)key
{
    NXEvent *event = [NXApp currentEvent];
    Matrix *matrix = [browser matrixInColumn:[browser lastColumn]];
    int selectedRow = [matrix selectedRow];
    id cell;
    int rowCount = [matrix cellCount];

    if (rowCount == 0) return;

    if (event->flags & NX_SHIFTMASK)
    {
	// Select next cell responding non-nil to cellHashContentsSelector.
	int row, direction, endRow;

	switch (key)
	{
	  case NX_UP:
	    row = (selectedRow < 0) ? rowCount : selectedRow;
	    direction = -1;
	    endRow = -1;
	    break;
	  case NX_DOWN:
	    row = selectedRow;
	    direction = +1;
	    endRow = rowCount;
	    break;
	  default:
	    return;
	}
#if DEBUG
	fprintf(stderr, "-%s (shift-arrow) start row %d, direction %d, end row %d\n", SELNAME(_cmd), row, direction, endRow);
#endif
	while ((row = row + direction) != endRow)
	{
	    cell = [matrix cellAt:row:0];

	    if ((cellHasContentsSelector &&
		 [cell respondsTo:cellHasContentsSelector] &&
		 [cell perform:cellHasContentsSelector with:nil]) ||
		![cell isLeaf])
	    {
		[browser setPath:[cell stringValue]];
		if (sendActionOnSelect) [browser sendAction];
		break;
	    }
	}
    }
    else if (event->flags & (NX_ALTERNATEMASK|NX_CONTROLMASK))
    {
	// Select first/last cell.
	switch (key)
	{
	  case NX_UP:
	    cell = [matrix cellAt:0:0];
	    break;
	  case NX_DOWN:
	    cell = [matrix cellAt:(rowCount-1):0];
	    break;
	  default:
	    return;
	}
	[browser setPath:[cell stringValue]];
	if (sendActionOnSelect) [browser sendAction];
    }
    else
    {
	// Avoid wraparound.
	switch (key)
	{
	  case NX_UP:
	    if (selectedRow == 0) return;
	    break;
	  case NX_DOWN:
	    if (selectedRow == (rowCount - 1)) return;
	    break;
	}
	[browser keyDown:event];
    }
}

- (void)complete:sender
{
    if ([sender delegate] == textField)
    {
	Matrix *matrix = [browser matrixInColumn:[browser lastColumn]];
	int selectedRow = [matrix selectedRow];

	if (selectedRow >= 0)
	{
	    const char *selectedTitle = [[matrix cellAt:selectedRow:0] stringValue];
	    const char *stringValue = [textField stringValue];

	    if (!(stringValue && *stringValue) ||
		(strcmp(selectedTitle, stringValue) != 0 &&
		 strncasecmp(selectedTitle, stringValue, strlen(stringValue)) == 0))
	    {
		[textField setStringValue:selectedTitle];
		[textField selectText:self];
		[addButton setEnabled:NO];
		[removeButton setEnabled:YES];
		return;
	    }
	}
    }
    NXBeep();
}

// Incremental selection in browser via text field.

- textDidGetKeys:sender isEmpty:(BOOL)isEmpty
{
    // sender == field editor, its delegate == text field.
    id theTextField = [sender delegate];

    if (theTextField == textField)
    {
	[self selectItemInBrowser:(isEmpty ? "" : [theTextField stringValue])];
    }
    return self;
}

- textDidEnd:sender endChar:(unsigned short)endChar
{
    // sender == field editor, its delegate == text field.
    id theTextField = [sender delegate];

    if (theTextField == textField)
    {
#if DEBUG
	fprintf(stderr, "-textDidEnd: endChar = %#x\n", endChar);
#endif
	/* Hand arrow {up,down} keys to browser; complete if ESC. */
	switch (endChar)
	{
	  case NX_DOWN:
	  case NX_UP:
	    [self handleArrowKey:endChar];
	    return nil;

	  case NX_RETURN:
	    [self selectItemInBrowser:[theTextField stringValue]];
	    break;

	  case ESC:
	    [self complete:sender];
	    return nil;

	  default:
	    break;
	}
    }
    return self;
}

@end // BrowserHelper (Private)


@implementation BrowserHelper

- (NXBrowser *)browser		{ return browser; }
- (TextField *)textField	{ return textField; }
- (Button *)addButton		{ return addButton; }
- (Button *)removeButton	{ return removeButton; }

- (BOOL)sendActionOnSelect	{ return sendActionOnSelect; }

- (void)setSendActionOnSelect:(BOOL)aValue
{
    if (aValue != sendActionOnSelect)
    {
	sendActionOnSelect = aValue;
	[browser acceptArrowKeys:YES andSendActionMessages:sendActionOnSelect];
    }
}

- (SEL)cellHasContentsSelector	{ return cellHasContentsSelector; }

- (void)setCellHasContentsSelector:(SEL)aCellHasContentsSelector
{
    cellHasContentsSelector = aCellHasContentsSelector;
}

- add:sender
{
    const char *title = [textField stringValue];
    int lastColumn = [browser lastColumn];
    Matrix *matrix;
    int row;
    id delegate;

    [self selectItemInBrowser:title];

    matrix = [browser matrixInColumn:lastColumn];

#if DEBUG
    if (!matrix) fprintf(stderr, "-[%s %s]: matrix in column %d is nil!!!\n", [[self class] name], sel_getName(_cmd), lastColumn);
#endif
    row = [matrix selectedRow];
    if (row < 0 || strcasecmp([[matrix cellAt:row:0] stringValue], title) <= 0)
    {
	++row;
    }

    delegate = [browser delegate];
    if (delegate == nil ||
	![delegate respondsTo:
	    @selector(browser:willAddCellNamed:inMatrix:atRow:)] ||
	[delegate browser:browser willAddCellNamed:title
	    inMatrix:matrix atRow:row])
    {
	[self addCellNamed:title inMatrix:matrix atRow:row];

	if ([delegate respondsTo:
		@selector(browser:didAddCell:inMatrix:atRow:)])
	{
	    [delegate browser:browser didAddCell:[matrix cellAt:row:0]
		inMatrix:matrix atRow:row];
	}
    }
    [matrix sizeToCells];
    [browser displayColumn:lastColumn];
    [self selectItemInBrowser:title];

    return self;
}

- remove:sender
{
    const char *title = [textField stringValue];
    int lastColumn = [browser lastColumn];
    Matrix *matrix;
    int selectedRow;

    [self selectItemInBrowser:title];

    matrix = [browser matrixInColumn:lastColumn];

    selectedRow = [matrix selectedRow];
    if (selectedRow >= 0)
    {
	const char *selectedTitle = [[matrix cellAt:selectedRow:0] stringValue];

	if (strcmp(selectedTitle, title) == 0)
	{
	    id delegate = [browser delegate];

	    if (delegate == nil ||
		![delegate respondsTo:
		    @selector(browser:willRemoveCellInMatrix:atRow:)] ||
		[delegate browser:browser willRemoveCellInMatrix:matrix
		    atRow:selectedRow])
	    {
		[matrix removeRowAt:selectedRow andFree:YES];

		if ([delegate respondsTo:
			@selector(browser:didRemoveCellInMatrix:atRow:)])
		{
		    [delegate browser:browser didRemoveCellInMatrix:matrix
			atRow:selectedRow];
		}
	    }
	    [matrix sizeToCells];
	    [browser displayColumn:lastColumn];
	    [self selectItemInBrowser:title];
	}
    }
    return self;
}

- reload:sender
{
    [browser loadColumnZero];
    [self selectItemInBrowser:[textField stringValue]];

    return self;
}

- select:sender
{
    [textField selectText:sender];
    return self;
}

@end // BrowserHelper
