/*+++
 *  title:      DefaultsHandler.m
 *  abstract:   Implementation of DefaultsHandler support class for preferences.
 *  author:     Tom Hageman <tom@basil.icce.rug.nl>
 *  created:    March 1999
 *  modified:   (see RCS Log at end)
 *  copyleft:
 *
 *		Copyright (C) 1999  Tom R. Hageman.
 *
 *	This is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This software is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this software; see the file COPYING.  If not, write to
 *	the Free Software Foundation, 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 *
 *  description:
 *      (see corresponding *.h file)
 *---*/

#ifdef RCS_ID
static const char * const RCSid = ((void)&RCSid,
	"@(#)DefaultsHandler.m,v 1.5 1999/04/08 18:54:02 tom Exp");
#   define RCS_DefaultsHandler_ID
#endif

#import "DefaultsHandler.h"

#import <appkit/appkit.h>
#import <ctype.h>


#define MAX_LINE_LEN 8096	// Maximum length of one preferences entry

#define GLOBAL_NX		"GLOBAL"
#define GLOBAL_OS		"NSGlobalDomain"


@implementation DefaultsHandler

- delegate { return delegate; }
- (void)setDelegate:aDelegate
{
    delegate = aDelegate;
}

- (NXAtom)nextstepDefaultName { return nextstepDefaultName; }
- (void)setNextstepDefaultName:(const char *)aName
{
    nextstepDefaultName = NXUniqueString(aName);
    if (!openstepDefaultName) openstepDefaultName = nextstepDefaultName;
}

- (NXAtom)openstepDefaultName { return openstepDefaultName; }
- (void)setOpenstepDefaultName:(const char *)aName
{
    openstepDefaultName = NXUniqueString(aName);
    if (!nextstepDefaultName) nextstepDefaultName = openstepDefaultName;
}

- (NXAtom)nextstepGlobalOwner { return NXUniqueString(GLOBAL_NX); }
- (NXAtom)openstepGlobalOwner { return NXUniqueString(GLOBAL_OS); }

// support methods.

- (const char *)bundleName
{
    return strrchr([[NXBundle bundleForClass:[self class]] directory], '/') + 1;
}

- (BOOL)getPath:(char *)aBuf forResource:(const char *)aResource
{
    NXBundle *bundle = [NXBundle bundleForClass:[self class]];
    BOOL result = [bundle getPath:aBuf forResource:aResource ofType:NULL];

    if (!result)
    {
	NXLogError("%s: unable to find \"%s\" in bundle.", [self bundleName], aResource);
	NXBeep();
    }
    return result;
}

/* To circumvent 8-bit dirty sh/system(), also \octal encode 8-bit
   characters.  This only works for OPENSTEP defaults though,
   which is OK I guess since we now handle NEXTSTEP internally anyway... */

static const char *shell_escape(char *buf, const char *aValue, BOOL escape8bit)
{
    const char *s = aValue;
    char *d = buf;
    unsigned char c;

    *d++ = '\'';
    while ((c = *s++))
    {
	if (c == '\'')
	{
	    *d++ = c;
	    *d++ = '\\';
	    *d++ = c;
	    *d++ = c;
	    continue;
	}
	if (c == '\\' || c == '"')
	{
	    *d++ = '\\';
	    *d++ = c;
	    continue;
	}
	if ((c & 0x80) && escape8bit)
	{
	    sprintf(d, "\\%03o", c);
	    d += 4;
	    continue;
	}
	*d++ = c;
    }
    *d++ = '\'';
    *d = '\0';

    return buf;
}

- (BOOL)setOwner:(const char *)anOwner
    value:(const char *)aValue
    nextstep:(BOOL)nextstep
    openstep:(BOOL)openstep
    /* set a user default value for owner `anOwner' for NEXTSTEP and/or
       OPENSTEP (with the implicit defaultName keys.)
       Removes the user default if aValue = NULL. */
{
    BOOL result = YES;
    int len;

    len = MAXPATHLEN+1+50;	// allow some slop for options, whitespace etc.

    if (!anOwner) anOwner = GLOBAL_NX;

    if (nextstep)
    {
	// NEXTSTEP defaults can be handled internally.
	if (aValue)
	{
	    result = NXWriteDefault(anOwner, nextstepDefaultName, aValue);
	}
	else
	{
	    result = NXRemoveDefault(anOwner, nextstepDefaultName);
	}
	if (!openstep) return result;
    }

    // OPENSTEP defaults are handled by the setpref.sh script.
    len += strlen(anOwner);
/// if (nextstep) len += strlen(nextstepDefaultName);
    if (openstep) len += strlen(openstepDefaultName);
    if (aValue) len += strlen(aValue) * 4; // max expansion for shell-escape.
  {
    char command[len];

    if (![self getPath:command forResource:"setpref.sh"]) return 0;

    if (!aValue) strcat(command, " -r");
    strcat(command, " "), strcat(command, anOwner);
/// if (nextstep) strcat(command, " -N "), strcat(command, nextstepDefaultName);
    if (openstep) strcat(command, " -O "), strcat(command, openstepDefaultName);
    if (aValue) strcat(command, " "), shell_escape(command+strlen(command), aValue, YES);

    if (system(command) != 0) result = NO;
  }
    return result;
}


static const char *decode_octal_escapes(char *buf, const char *aValue)
{
    const char *s = aValue;
    char *d = buf;
    unsigned char c;

    while ((c = *s++))
    {
	if (c == '\\')
	{
	    if (isdigit(*s))
	    {
		// Brute-force octal decode, not cool but gets the job done...
		c = *s++ - '0';
		if (isdigit(*s))
		{
		    c = (c << 3) + (*s++ - '0');
		    if (isdigit(*s))
		    {
			c = (c << 3) + (*s++ - '0');
		    }
		}
	    }
	    else if (*s != '\0')
	    {
		// Just simple single-char escape.
		c = *s++;
	    }
	}
	*d++ = c;
    }
    *d = '\0';

    return buf;
}

- (int)scanPreferences:(void *)context
    /* scan through a list of all user defaults, XXX */
{
    char command[MAXPATHLEN+1+strlen(nextstepDefaultName)+1+strlen(openstepDefaultName)+1];
    char lines[3][MAX_LINE_LEN+1];
    int lineSel = 0;
    char *line;
    FILE *fp;
    const char *kind, *owner, *variable, *value;
    NXAtom newOwner;
    int count;
    struct {
	NXAtom owner;
	const char *nextstepValue;
	const char *openstepValue;
    } set;

    if (![self getPath:command forResource:"scanprefs.sh"]) return -1;

    sprintf(command+strlen(command), " %s %s", nextstepDefaultName, openstepDefaultName);

    /* Call the 'scanprefs.sh' command as a subprocess
       This returns a uniqued list of the form:
         <kind> <owner>
       or
         <kind> <owner> <key> <value>
       where <kind> is either `N' for NEXTSTEP, or `O' for OPENSTEP.
     */
    fp = popen(command, "r");
    if (!fp)
    {
	NXLogError("%s: unable to popen(\"%s\")", [self bundleName], command);
	NXBeep();
	return -1;
    }

    // Get ready to read, erase previous information
    set.owner = NULL;
    count = 0;

    // Process the subprocess output, line by line
    /* XXX display will be inaccurate if NEXTSTEP and OPENSTEP versions
       have different defaults for the same owner.  In that case it
       uses the last value encountered (which happens to be OPENSTEP
       due to the way the list is sorted by scanprefs.sh. */

    for (;  fgets((line = lines[lineSel]), MAX_LINE_LEN, fp);  lineSel = (lineSel + 1) % 3)
    {
	// Isolate the first two fields
	if (!(kind = strtok(line, " \"\n\t"))) continue;
	if (!(owner = strtok(NULL, " \"\n\t"))) continue;

	// New owner found?
	newOwner = NXUniqueString(owner);

	if (newOwner != set.owner)
	{
	    // Report previous owner and its value to delegate.
	    if (set.owner)
	    {
		if ([delegate defaultsHandler:self
			foundOwner:set.owner
			nextstepValue:set.nextstepValue
			openstepValue:set.openstepValue
			context:context]) count++;
	    }

	    // Set up this entry
	    set.owner = newOwner;
	    set.nextstepValue = NULL;
	    set.openstepValue = NULL;
	}

	switch (*kind)
	{
	  case 'N':
	    if (!set.nextstepValue) set.nextstepValue = "";
	    break;
	  case 'O':
	    if (!set.openstepValue) set.openstepValue = "";
	    break;
	}

	// Does it have a value?
	if (!(variable = strtok(NULL, " \n\t"))) continue;

	// (scanprefs should only return those keys we asked for.)

	// Yes, find the setting
	if (!(value = strtok(NULL, "\n"))) continue;

	switch (*kind)
	{
	  case 'N':
	    set.nextstepValue = value;
	    break;
	  case 'O':
	    // For OPENSTEP, de-\octal escape the value string.
	    // (Actually should do this for entire line...)
	    // NB can be done in-place.
	    set.openstepValue = decode_octal_escapes((char *)value, value);
	    break;
	}
    }

    // Write the last owner
    if (set.owner)
    {
	if ([delegate defaultsHandler:self
		foundOwner:set.owner
		nextstepValue:set.nextstepValue
		openstepValue:set.openstepValue
		context:context]) count++;
    }

    // Deallocate resources
    pclose(fp);

    return count;
}

@end // DefaultsHandler

