/* Copyright (c) 1992 by NightShade Software.
 *
 * NAME
 *     WindowDepth : Layout
 *
 * DESCRIPTION
 *     A custom Preferences option for setting WindowDepthLimits.
 *
 * COMMENTS/PROBLEMS
 *     ---
 *
 * AUTHOR/DATE CREATED
 *     Gary Ritchie/November 24, 1992
 *
 * REVISIONS
 *     Tom Hageman -- Nov 1997 -- OPENSTEP support.
 *                 -- Mar 1999 -- cleanup + support for adding new owners.
 */

#ifdef RCS_ID
static const char * const RCSid = ((void)&RCSid,
	"@(#)WindowDepth.m,v 1.13 1999/04/18 10:36:57 tom Exp");
#   define RCS_WindowDepth_ID
#endif

#import "WindowDepth.h"

#import "WindowDepthBrowserCell.h"
#import "BrowserHelper.h"
#import "DefaultsHandler.h"

#import "common.h"
#import "version.h"

#define WINDOWDEPTHLIMIT_NX	"NXWindowDepthLimit"
#define WINDOWDEPTHLIMIT_OS	"NSWindowDepthLimit"

/* Names used as valuse in defaults database (non-test mode). */
static const char *const DEFAULT_NAMES[] = {
    "TwoBitGray",
    "EightBitGray",
    "TwelveBitRGB",
    "TwentyFourBitRGB",
    NULL
};

static const char TEST_PREFIX[] = "Test";


/* Used to store temporary information */
typedef struct {
    NXAtom owner;		// The name of the application
    int depth;			// The WindowDepthLimit
    BOOL test;			// YES if using Test prefix
    BOOL nextstep;		// YES if NEXTSTEP default
    BOOL openstep;		// YES if OPENSTEP default (can be both).
} AppInfo;


@interface WindowDepth(Private)
- setBrowserCellTo:(AppInfo *)set inMatrix:matrix atRow:(int)row;
@end

@implementation WindowDepth(Private)

/*=========================================================================
 * NAME        : setBrowserCell:withMatrix:
 * DESCRIPTION : Insert a new cell in the browser.
 *=========================================================================*/
- setBrowserCellTo:(AppInfo *)set inMatrix:matrix atRow:(int)row
{
    id cell;

    [matrix insertRowAt:row];
    cell = [matrix cellAt:row :0];
    [cell setLoaded:YES];
    [cell setLeaf:YES];

    [cell setOwner:set->owner];
    [cell setDepth:set->depth];
    [cell setTestMode:set->test];
    [cell setNextstep:set->nextstep];
    [cell setOpenstep:set->openstep];
    return self;
}

@end // WindowDepth(Private)


@implementation WindowDepth

/*=========================================================================
 * NAME        : init
 * DESCRIPTION : Called first time this section's button is clicked.
 *=========================================================================*/
- init
{
    self = [super init];

    if (![NXApp loadNibForLayout:[[self class] name] owner:self]) return nil;

    [infoVersionTextField setStringValue:VERSION" ("VERSION_DATE")"];

    // 'window' is an interface builder outlet.
    view = [window contentView];	// 'view' MUST be initialized
    [view removeFromSuperview];
    [window setContentView: 0];		// Don't need the window for anything
    [window free];			// So free it.

    // When multiple items (or none) are selected, no radio button is set
    [appRadio setEmptySelectionEnabled:YES];
    [appRadio clearSelectedCell];

    // Browser uses a custom Cell
    [[browserHelper browser] setCellClass:[WindowDepthBrowserCell class]];
    [browserHelper setCellHasContentsSelector:@selector(depthImage)];

    return self;
}

/*=========================================================================
 * NAME        : willSelect:
 * DESCRIPTION : Before the view is added via addSubview.
 *=========================================================================*/
- willSelect: sender
{
    return self;
}

/*=========================================================================
 * NAME        : didSelect:
 * DESCRIPTION : After the view is added via addSubview.
 *=========================================================================*/
- didSelect: sender
{
    // Enable relevant menu items
    [NXApp enableEdit:COPY_ITEM];
    [NXApp enableWindow:CLOSE_ITEM];

    // Set up defaults handler.
    if (!defaultsHandler)
    {
	defaultsHandler = [[DefaultsHandler alloc] init];
	[defaultsHandler setDelegate:self];
	[defaultsHandler setNextstepDefaultName:WINDOWDEPTHLIMIT_NX];
	[defaultsHandler setOpenstepDefaultName:WINDOWDEPTHLIMIT_OS];
    }

    // Rebuild the list of apps and preferences
    [self rescanPrefs:self];
    [browserHelper select:self];

    return self;
}


/*=========================================================================
 * NAME        : willUnselect:
 * DESCRIPTION : Before removeFromSuperview
 *=========================================================================*/
- willUnselect: sender
{
    // Make sure that we aren't editing any text fields.
    [[NXApp appWindow] endEditingFor:self];
    return self;
}

/*=========================================================================
 * NAME        : didUnselect:
 * DESCRIPTION : Before removeFromSuperview
 *=========================================================================*/
- didUnselect: sender
{
    // Hide our info panel
    [infoPanel orderOut:self];

    // Disable all Edit and Window menu items
    [NXApp enableEdit: NO];
    [NXApp enableWindow: NO];
    return self;
}

/*=========================================================================
 * NAME        : didHide:
 * DESCRIPTION : Application was just hidden.
 *=========================================================================*/
- didHide: sender
{
    return self;
}

/*=========================================================================
 * NAME        : didUnhide:
 * DESCRIPTION : Application was just unhidden.
 *=========================================================================*/
- didUnhide: sender
{
    [browserHelper select:self];
    return self;
}

/*=========================================================================
 * NAME        : rescanPrefs:
 * DESCRIPTION : Rescan the user's preferences.
 *=========================================================================*/
- rescanPrefs:sender
{
#if DEBUG
    fprintf(stderr, "-rescanPrefs:\n");
#endif
    // Cause the browser to reload
    [browserHelper reload:self];

    return self;
}

/*=========================================================================
 * NAME        : itemSelected:
 * DESCRIPTION : Target of NXBrowser.
 *=========================================================================*/
- itemSelected:sender
{
    List *aList = [[List alloc] init];
    int i, count;

    // Get a list of all the selected cells in the browser
    [sender getSelectedCells:aList];
    count = [aList count];

    // Set the radio button and switch to match the selection
    if (count == 0)
    {
        [appRadio clearSelectedCell];
	[appRadio display];
	[appSwitch setState:0];
        [appRadio setEnabled:NO];
	[appSwitch setEnabled:NO];
    }
    else
    {
        id cell = [aList objectAt:0];
	int depth = [cell depth];
	BOOL testMode = [cell testMode];

        [appRadio selectCellWithTag:depth];
	[appSwitch setState:testMode];
	for (i = 1;  i < count;  i++)
	{
	    cell = [aList objectAt:i];
	    if ([cell depth] != depth || [cell testMode] != testMode)
	    {
		[appRadio clearSelectedCell];
		[appRadio display];
		[appSwitch setState:0];
		break;
	    }
	}
	[appRadio setEnabled:YES];
	[appSwitch setEnabled:YES];
    }
    [aList free];
    return self;
}

/*=========================================================================
 * NAME        : browser:fillMatrix:inColumn:
 * DESCRIPTION : Delegate message from browser requesting a reload.
 *=========================================================================*/

typedef struct {
    int row;
    id matrix;
    int globalDepth;
    BOOL globalTestMode;
    NXAtom globalNX;	// cached for speed.
    NXAtom globalOS;
} ScanPrefsContext;

- (int)browser:sender fillMatrix:matrix inColumn:(int)column
{
    ScanPrefsContext context;

#if DEBUG
    fprintf(stderr, "-browser:fillMatrix:inColumn: ...\n");
#endif
    context.row = 0;
    context.matrix = matrix;
    context.globalDepth = USE_DEFAULT;
    context.globalTestMode = NO;
    context.globalNX = [defaultsHandler nextstepGlobalOwner];
    context.globalOS = [defaultsHandler openstepGlobalOwner];

    // Scan preferences; fill browser using callback.
    if ([defaultsHandler scanPreferences:&context] < 0) return 0;

    // Set the radio buttons for the GLOBAL setting
    [defaultRadio selectCellWithTag:context.globalDepth];
    [defaultRadio display];
    [defaultSwitch setState:context.globalTestMode];

    // Nothing is currently selected in the browser
    [appRadio clearSelectedCell];
    [appSwitch setState:0];
    [appRadio setEnabled:NO];
    [appSwitch setEnabled:NO];

#if DEBUG
    fprintf(stderr, "-browser:fillMatrix:inColumn: ...(done)\n");
    fprintf(stderr, "  (global.depth = %d, global.test = %d, rows = %d)\n", context.globalDepth, context.globalTestMode, context.row);
#endif
    return context.row;
}

// DefaultsHandler -scanPreferences callback.

- (BOOL)defaultsHandler:sender
    foundOwner:(NXAtom)anOwner
    nextstepValue:(const char *)nextstepValue
    openstepValue:(const char *)openstepValue
    context:(void *)aContext
{
    ScanPrefsContext *context = aContext;
    AppInfo set;
    const char *aValue;

    set.owner = anOwner;
    set.depth = USE_DEFAULT;
    set.test = NO;
    set.nextstep = (nextstepValue != NULL);
    set.openstep = (openstepValue != NULL);

    // OPENSTEP value takes precedence over NEXTSTEP if these are both defined.
    aValue = ((openstepValue && *openstepValue) ? openstepValue :
	      (nextstepValue && *nextstepValue) ? nextstepValue :
	      NULL);

    if (aValue)
    {
	int i;

	if (strncmp(aValue, TEST_PREFIX, sizeof(TEST_PREFIX)-1) == 0)
	{
	    set.test = YES;
	    aValue += sizeof(TEST_PREFIX)-1;
	}
	for (i = 0;  i < USE_DEFAULT;  i++)
	{
	    if (strcmp(aValue, DEFAULT_NAMES[i]) == 0) break;
	}
	set.depth = i;
    }

    if (anOwner == context->globalNX || anOwner == context->globalOS)
    {
	if (set.depth != USE_DEFAULT)
	{
	    context->globalDepth = set.depth;
	    context->globalTestMode = set.test;
	}
	return NO;
    }

    [self setBrowserCellTo:&set inMatrix:context->matrix atRow:context->row++];

    return YES;
}

/*=========================================================================
 * NAME        : setDefault:
 * DESCRIPTION : Target of Default radio button matrix.
 *=========================================================================*/

static const char *prefDepthName(char *scratch, int depth, BOOL testMode)
{
    const char *result = DEFAULT_NAMES[depth];

    if (result && testMode)
    {
	sprintf(scratch, "%s%s", TEST_PREFIX, result); 
	result = scratch;	
    }
    return result;
}

- setDefault:sender
{
    char scratch[100];
    int depth = [[defaultRadio selectedCell] tag];
    BOOL test = [defaultSwitch state];
    const char *value = prefDepthName(scratch, depth, test);

    [defaultsHandler setOwner:NULL value:value nextstep:YES openstep:YES];

    return self;
}

/*=========================================================================
 * NAME        : setApp:
 * DESCRIPTION : Target of Application radio button matrix.
 *=========================================================================*/
- setApp:sender
{
    char scratch[100];
    int depth = [[appRadio selectedCell] tag];
    BOOL test = [appSwitch state];
    const char *value = prefDepthName(scratch, depth, test);
    id browser = [browserHelper browser];
    List *aList;
    int i, count;

    if (depth == USE_DEFAULT) test = NO;

    // Get a list of all the selected cells in the browser
    aList = [[List alloc] init];
    [browser getSelectedCells:aList];

    // Modify the defaults database to match
    count = [aList count];
    for (i = 0;  i < count;  i++)
    {
	id cell = [aList objectAt:i];
	NXAtom owner = [cell owner];

	[defaultsHandler setOwner:owner value:value
	    nextstep:[cell nextstep]
	    openstep:[cell openstep]];

	[cell setOwner:owner];	// needed?
	[cell setLoaded:YES];	// needed?
	[cell setDepth:depth];
	[cell setTestMode:test];
    }

    [aList free];
    [browser displayColumn:[browser lastColumn]];

    return self;
}

@end // WindowDepth
