/* Copyright (c) 1992 by NightShade Software
 *
 * NAME
 *     WindowDepthBrowserCell : NXBrowserCell
 *
 * DESCRIPTION
 *     Custom BrowserCell for WindowDepth preferences module.
 *
 * COMMENTS/PROBLEMS
 *     ----
 *
 * AUTHOR/DATE CREATED
 *     Gary Ritchie/November 24, 1992
 *
 * REVISIONS
 *     Tom Hageman -- Nov 1997 -- OPENSTEP Support
 *                 -- Mar 1999 -- rename class to avoid potential conflicts.
 */
 
#import "WindowDepthBrowserCell.h"
#import "common.h"

#define FIRST_COLUMN_START 5.0
#define SECOND_COLUMN_START 45.0


@implementation WindowDepthBrowserCell

static NXImage *images[NR_DEPTH_VALUES];
static NXSize	imageSize;		// All assumed to be same size

static NXImage *loadImage(const char *imageName, NXBundle *bundle)
{
    if (imageName)
    {
	char buf[MAXPATHLEN+1];

	if ([bundle getPath:buf forResource:imageName ofType:"tiff"])
	{
	    return [[NXImage alloc] initFromFile:buf];
	}
    }
    return nil;
}

+ initialize
{
    if (!images[USE_2_BIT])
    {
	id bundle = [NXBundle bundleForClass:[self class]];
	int i;

	for (i = 0;  i < sizeof(images)/sizeof(images[0]);  i++)
	{
	    const char *tiff = NULL;

	    switch (i)
	    {
	      case USE_2_BIT:	tiff = "2";	break;
	      case USE_8_BIT:	tiff = "8";	break;
	      case USE_12_BIT:	tiff = "12";	break;
	      case USE_24_BIT:	tiff = "24";	break;
	    }
	    images[i] = loadImage(tiff, bundle);
	}
	[images[USE_2_BIT] getSize:&imageSize];
    }
   return self;
}

/*=========================================================================
 * NAME        : initTextCell:
 * DESCRIPTION : Initialize a cell.
 *=========================================================================*/
- initTextCell:(const char *)aString
{
    self = [super initTextCell:aString];

    if (self)
    {
	// provide default values for ivars.
	depth = USE_DEFAULT;
	test = NO;
	nextstep = YES;
	openstep = YES;
    }
    return self;
}

/*=========================================================================
 * NAME        : setOwner:
 * DESCRIPTION : Set the owner attribute.
 *=========================================================================*/
- setOwner:(NXAtom)theOwner
{
    [self setStringValueNoCopy:theOwner];
    owner = theOwner;
    return self;
}

/*=========================================================================
 * NAME        : owner
 * DESCRIPTION : Return the owner attribute.
 *=========================================================================*/
- (NXAtom)owner
{
    if (!owner)
    {
	const char *stringValue = [self stringValue];

	if (stringValue) owner = NXUniqueString(stringValue);
    }
    return owner;
}

/*=========================================================================
 * NAME        : setDepth:
 * DESCRIPTION : Set the WindowDepthLimit attribute.
 *=========================================================================*/
- setDepth:(int)theDepth;
{
    depth = theDepth;
    return self;
}

/*=========================================================================
 * NAME        : depth
 * DESCRIPTION : Return the WindowDepthLimit.
 *=========================================================================*/
- (int)depth
{
    return depth;
}

- (NXImage *)depthImage
{
    return ((unsigned)depth < sizeof(images)/sizeof(images[0]) ? images[(int)depth] : nil);
}

/*=========================================================================
 * NAME        : setTestMode:
 * DESCRIPTION : Sets the test flag.
 *=========================================================================*/
- setTestMode:(BOOL)flag
{
    test = flag;
    return self;
}

/*=========================================================================
 * NAME        : testMode
 * DESCRIPTION : Returns the state of the test mode flag.
 *=========================================================================*/
- (BOOL)testMode
{
    return test;
}

// [TRH 10-Nov-97] Access methods for nextstep/openstep defaults flags.
- setNextstep:(BOOL)flag	{ nextstep = flag;  return self; }
- (BOOL)nextstep		{ return nextstep; }
- setOpenstep:(BOOL)flag	{ openstep = flag;  return self; }
- (BOOL)openstep		{ return openstep; }

/*===========================================================================
 * NAME        : - drawInside:inView:
 * DESCRIPTION : Draw the custom cell.
 *===========================================================================*/
- drawInside:(const NXRect *)cellFrame inView:controlView
{
    static NXCoord ascender, descender, lineHeight;
    static Font *lastFont;

    NXPoint	imageOrigin;
    NXImage	*depthImage;
    Font	*font;

    [super drawInside:cellFrame inView:controlView];
    
    /* erase the cell */
    PSsetgray((cFlags1.highlighted|cFlags1.state) ? NX_WHITE : NX_LTGRAY);
    NXRectFill(cellFrame);
    
    /* draw the depth tiff */
    if ((depthImage = [self depthImage]))
    {
        imageOrigin.x = FIRST_COLUMN_START;
	imageOrigin.y = NX_Y(cellFrame) + NX_HEIGHT(cellFrame) -
	                (NX_HEIGHT(cellFrame) - imageSize.height) / 2.0;
	[depthImage composite:NX_SOVER toPoint:&imageOrigin];
    }
    
    /* draw the text */
    font = support; //[self font];

    //PSsetgray((depth != USE_DEFAULT) ? NX_BLACK : NX_DKGRAY);
    PSsetgray(NX_BLACK);

    if (depth != USE_DEFAULT)
    {
	font = [[FontManager new] convert:font toHaveTrait:NX_BOLD];
    }
    if (test)
    {
	font = [[FontManager new] convert:font toHaveTrait:NX_ITALIC];
    }
    if (lastFont != font)
    {
	lastFont = font;
	NXTextFontInfo(font, &ascender, &descender, &lineHeight);
    }
    if (NXDrawingStatus == NX_DRAWING) // ...so much for WYSIWIG.
    {
	font = [font screenFont];
    }
    [font set];

    PSmoveto(NX_X(cellFrame) + SECOND_COLUMN_START,
    	     NX_Y(cellFrame) + lineHeight - descender);

    PSshow(contents);

    // Mark OPENSTEP owners.
    if (openstep)
    {
	static TextFieldCell *sharedTextCell;
	NXSize sharedTextCellSize;
	NXRect rect = (*cellFrame);

	//make cell
	if (!sharedTextCell)
	{
	    sharedTextCell = [[TextFieldCell alloc] init];
	    [sharedTextCell setWrap:NO];
	    [sharedTextCell setFont:[self font]];
	    [sharedTextCell setAlignment:NX_RIGHTALIGNED];
	}

	[sharedTextCell setStringValueNoCopy:(nextstep ? "n,o" : "o")];
	[sharedTextCell calcCellSize:&sharedTextCellSize];

	NX_X(&rect) = NX_MAXX(&rect) - sharedTextCellSize.width + 1;
	NX_WIDTH(&rect) = sharedTextCellSize.width;

	PSsetgray((cFlags1.highlighted|cFlags1.state) ? NX_WHITE : NX_LTGRAY);
	NXRectFill(&rect);

	[sharedTextCell drawInside:&rect inView:controlView];	
    }
    return self;
}

@end // WindowDepthBrowserCell
